/*==========================================================================;
 *
 *  File:       entrypoint.cpp
 *  Content:    GL initialization code
 *
 ***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <GL/gl.h>
#include <GL/glx.h>
#include <X11/Xlib.h>

#include "../gl_init.h"

#include "../d3dprj.h"
#include "configuration.h"

struct gl_driver_information
{
  Display *display;
  Window window;
  int screen;
  XVisualInfo *visual;
  GLXContext context;
  HWND fenetre_locale;
};

struct handle_cast
{
  Display *display;
  Window window;
};

int attrSingleBuffer[] = {GLX_RGBA, GLX_RED_SIZE, 4, 
			  GLX_GREEN_SIZE, 4, 
			  GLX_BLUE_SIZE, 4, 
			  GLX_DEPTH_SIZE, 16,
			  None};

int attrListDbl[] = { GLX_RGBA, GLX_DOUBLEBUFFER, 
		      GLX_RED_SIZE, 4, 
		      GLX_GREEN_SIZE, 4, 
		      GLX_BLUE_SIZE, 4, 
		      GLX_DEPTH_SIZE, 16,
		      None };

void lire_adresse(char *nom, int *width, int *height)
{
	FILE *f = fopen("d3d8config.txt","ra");
	int i = 0;
	char c;
	if (!f)
	{
		fprintf(stderr,"cannot open the configuration file\n");
		exit(1);
	}
	do
	{
		fread(&c,1,1,f);
		nom[i] = c;
		i++;
	}
	while (c != '\n');
	i--;
	nom[i] = 0;

	if (!feof(f))
	{
		fscanf(f,"%d*%d\n",width,height);
	}
	fclose(f);
}

int GL_CreateDevice(struct gl_driver_information **pDevice, HWND hwnd, int index, int width, int height)
{
  int _width = 0, _height = 0;
  char adresse[30];
  XSetWindowAttributes attr;
  gl_driver_information *p = new gl_driver_information;
  
  if (!p)
    return -4;

  lire_adresse(adresse,&_width,&_height);
  *pDevice = 0;
  handle_cast *h = (handle_cast *) hwnd;
  
  //p->display = h->display;
  //p->window = h->window;
  p->fenetre_locale = hwnd;
  p->display = XOpenDisplay(adresse);
  if (!p->display)
  {
	  fprintf(stderr,"Cannot open display");
	  system("pause");
	  exit(1);
  }

  p->screen = DefaultScreen(p->display);

  p->visual = glXChooseVisual(p->display,p->screen,attrListDbl);
  p->context = glXCreateContext(p->display, p->visual, 0, GL_FALSE);
  attr.colormap = XCreateColormap(p->display, RootWindow(p->display,p->screen),p->visual->visual,AllocNone);
  attr.border_pixel = 0;

	if (_width && _height)
	{
	}
	else
	{
		_width = localWindowWidth(p->fenetre_locale);
		_height = localWindowHeight(p->fenetre_locale);
	}
 
  p->window = XCreateWindow(p->display,RootWindow(p->display,p->screen),0,0,_width,_height,
							0,p->visual->depth,InputOutput,p->visual->visual,CWBorderPixel | CWColormap,&attr);


  if (!glXMakeCurrent(p->display, p->window, p->context))
  {
	  fprintf(stderr,"failure in glXMakeCurrent");
	  system("pause");
	  exit(1);
  }


  XMapWindow(p->display,p->window);

  *pDevice = p;
  
  GL_DefaultSettings(_width, _height);	
  return 0;
}

void GL_ReleaseDevice(struct gl_driver_information *pDevice)
{
  glXMakeCurrent(pDevice->display, 0, 0);
  if (pDevice)
    {
      glXDestroyContext(pDevice->display, pDevice->context);
      pDevice->context = 0;
    }
  XCloseDisplay(pDevice->display);
  pDevice->display = 0;
}

void GL_EnumAdapterModes(struct gl_driver_information *pDevice, int index, UINT *width, UINT *height, UINT *bpp, UINT *refreshRate)
{
}

void GL_GetAdapterDisplayMode(struct gl_driver_information *pDevice, UINT *width, UINT *height, UINT *bpp, UINT *refreshRate)
{
}

int GL_GetAdapterModeCount(struct gl_driver_information *pDevice)
{
  return 0;
}

int GL_GetAdapterCount(struct gl_driver_information *pDevice)
{
  return 1;
}

void GL_GetAdapterName(struct gl_driver_information *pDevice, int index, char *s, int numBytes)
{
  if (pDevice->window)
    strncpy(s,(char*)glGetString(GL_RENDERER),numBytes);
  else
    strncpy(s,"OpenGL Layer Display" ,numBytes);
  return;
}

int GL_CreatePseudoDevice(struct gl_driver_information **pDevice)
{
  gl_driver_information *p = new gl_driver_information;
  *pDevice = 0;

  if (!p)
    return -4;
  memset(p, 0, sizeof(gl_driver_information));
  p->display = 0;
  
  *pDevice = p;
  return 0;
}

void GL_PresentDevice(struct gl_driver_information *pDevice, void *h)
{
  glXSwapBuffers(pDevice->display,pDevice->window);
  return;
}

void GL_DefaultSettings(int width, int height)
{
  glEnable(GL_CULL_FACE);
  glCullFace(GL_FRONT);
  
  glMatrixMode( GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0., (float)width, (float)height, 0., -1., 1.);

  glMatrixMode( GL_MODELVIEW );
  glLoadIdentity();
  
  glViewport(0, 0, width, height);
  
  glPixelStorei( GL_UNPACK_ALIGNMENT, 1);
  glAlphaFunc( GL_GEQUAL, 0.5f);
}

void GL_GetCreationParameters(struct gl_driver_information *pDevice, struct _D3DDEVICE_CREATION_PARAMETERS *p)
{
  p->AdapterOrdinal = D3DADAPTER_DEFAULT ;
  p->BehaviorFlags = D3DCREATE_PUREDEVICE | D3DCREATE_MULTITHREADED | D3DCREATE_HARDWARE_VERTEXPROCESSING ;
  p->DeviceType = D3DDEVTYPE_HAL;
  //  p->hFocusWindow = pDevice->hwnd;
}

void DisplaySplashScreen(int sdk)
{
  fprintf(stderr,"Loading D3D Wrapper...\n");
}

void *glAllocateMemory(int size)
{
	return new BYTE[size];
}

void glFreeMemory(void *ptr)
{
  //	delete [] ptr;
}

void MESG(const char *pszFormat, ...)
{
	char s[128];
	va_list va;
	va_start(va, pszFormat);
	vsprintf(s, pszFormat, va);
	va_end(va);
	fprintf(stderr,s);
	

	FILE *fp;
	fp = fopen("d3d8.log","a+t");
	if (fp)
	{		
		fprintf(fp,"%s",s);
		fclose(fp);
	}
}

const char *GL_GetErrorString(GLenum err)
{
	switch(err)
	{
		case GL_NO_ERROR: return "NO ERROR"; 
		case GL_INVALID_ENUM: return "INVALID ENUM"; 
		case GL_INVALID_OPERATION: return "INVALID OPERATION"; 		
		case GL_STACK_UNDERFLOW: return "STACK UNDERFLOW"; 
		case GL_STACK_OVERFLOW: return "STACK OVERFLOW"; 
		case GL_OUT_OF_MEMORY: return "OUT OF MEMORY"; 
	}
	return "UNDEFINED";
}
